import { render, screen, waitFor } from '@testing-library/react-native'
import React from 'react'

import PosthogFeatureFlagGuard from '@holi/core/components/PosthogFeatureToggleGuard'
import { FeatureFlagKey } from '@holi/core/helpers/useFeatureFlag.common'
import HoliText from '@holi/ui/components/atoms/HoliText'

const mockUseFeatureFlag = jest.fn()
jest.mock('@holi/core/helpers/useFeatureFlag', () => ({
  useFeatureFlag: (flag: FeatureFlagKey) => mockUseFeatureFlag(flag),
}))

const mockFeatureFlags = (on: FeatureFlagKey[], off: FeatureFlagKey[] = [], maintenance: FeatureFlagKey[] = []) => {
  mockUseFeatureFlag.mockImplementation((flag: FeatureFlagKey) => {
    if (on.includes(flag)) return ['on', undefined]
    else if (off.includes(flag)) return ['off', undefined]
    else if (maintenance.includes(flag)) return ['maintenance', undefined]
    return undefined
  })
}

describe('FeatureToggleGuard', () => {
  it('should not render the feature component for unsupported key', async () => {
    const featureText = 'secret-feature-flag'
    mockFeatureFlags([FeatureFlagKey.DONATIONS], [], [])
    const unsupportedFeatureToggle = FeatureFlagKey.STORAGE

    render(
      <PosthogFeatureFlagGuard featureFlagKey={unsupportedFeatureToggle} i18nKey={'someFeatureNameKey'}>
        <HoliText>{featureText}</HoliText>
      </PosthogFeatureFlagGuard>
    )

    await waitFor(() => expect(screen.queryByText(featureText)).not.toBeOnTheScreen())
  })

  it('should render the feature component if flag set to "on"', async () => {
    const featureText = 'secret-feature-flag'
    mockFeatureFlags([FeatureFlagKey.CHAT])

    render(
      <PosthogFeatureFlagGuard featureFlagKey={FeatureFlagKey.CHAT} i18nKey={'chat.title'}>
        <HoliText>{featureText}</HoliText>
      </PosthogFeatureFlagGuard>
    )

    await waitFor(() => expect(screen.queryByText(featureText)).toBeOnTheScreen())
  })

  it('should not render the feature component if flag set to "off"', async () => {
    const featureText = 'secret-feature-flag'
    mockFeatureFlags([], [FeatureFlagKey.NOTIFICATIONS])

    render(
      <PosthogFeatureFlagGuard featureFlagKey={FeatureFlagKey.NOTIFICATIONS} i18nKey={'notifications.title'}>
        <HoliText>{featureText}</HoliText>
      </PosthogFeatureFlagGuard>
    )

    await waitFor(() => {
      expect(screen.queryByText(featureText)).not.toBeOnTheScreen()
    })
  })

  it('should render the maintenance mode component if flag set to "maintenance"', async () => {
    const featureText = 'secret-feature-flag'
    mockFeatureFlags([], [], [FeatureFlagKey.CHAT])

    render(
      <PosthogFeatureFlagGuard featureFlagKey={FeatureFlagKey.CHAT} i18nKey={'chat.title'}>
        <HoliText>{featureText}</HoliText>
      </PosthogFeatureFlagGuard>
    )

    expect(await screen.findByText('global.maintenance.description')).toBeOnTheScreen()
    expect(screen.queryByText(featureText)).not.toBeOnTheScreen()
  })
})
