import React, { memo } from 'react'
import { Trans, useTranslation } from 'react-i18next'
import { Linking, Platform, StyleSheet, View } from 'react-native'

import ScrollHeaderLayout from '@holi/core/components/ScrollHeaderLayout'
import { TrackingGeneralContentDescription } from '@holi/core/components/TrackingConsentModal'
import { getLogger } from '@holi/core/helpers/logging'
import useIsomorphicLayoutEffect from '@holi/core/helpers/useIsomorphicLayoutEffect'
import { HoliTextLink } from '@holi/core/navigation/components/HoliTextLink'
import { useSetScreenOptions } from '@holi/core/navigation/hooks/useScreenOptions'
import { updateConsentState, useConsentState } from '@holi/core/tracking/TrackingInitializer'
import { Consent, TrackingPurpose } from '@holi/core/tracking/types'
import HoliBox from '@holi/ui/components/atoms/HoliBox'
import HoliContainer from '@holi/ui/components/atoms/HoliContainer'
import { HoliGap } from '@holi/ui/components/atoms/HoliGap'
import { HoliHeading } from '@holi/ui/components/atoms/HoliHeading'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliLoader from '@holi/ui/components/molecules/HoliLoader'
import HoliSwitch from '@holi/ui/components/molecules/HoliSwitch'
import { HoliTheme, useTheme } from '@holi/ui/styles/theme'

const logger = getLogger('Tracking')

const isWeb = Platform.OS === 'web'

const updateUserConsentState = ({
  personalization = false,
  productAnalytics = false,
}: {
  personalization: boolean | null
  productAnalytics: boolean | null
}) => {
  updateConsentState({
    [TrackingPurpose.Personalization]: personalization ? Consent.Given : Consent.Denied,
    [TrackingPurpose.Analytics]: productAnalytics ? Consent.Given : Consent.Denied,
  })
}

const TrackingSettings = () => {
  const { t } = useTranslation()

  const { theme } = useTheme()
  const { colors } = theme
  const styles = getStyles(theme)

  const setScreenOptions = useSetScreenOptions()

  useIsomorphicLayoutEffect(() => {
    setScreenOptions({ title: t('profile.settings.tracking.title') })
  }, [setScreenOptions, t])

  const [consentState] = useConsentState()

  if (!consentState) return <HoliLoader />

  const {
    [TrackingPurpose.Personalization]: personalizationConsent = Consent.Unknown,
    [TrackingPurpose.Analytics]: productAnalyticsConsent = Consent.Unknown,
  } = consentState ?? {}
  const personalization = Consent.toBooleanOrNull(personalizationConsent)
  const productAnalytics = Consent.toBooleanOrNull(productAnalyticsConsent)

  const onAllPurposesChange = (value: boolean) => {
    updateUserConsentState({ personalization: value, productAnalytics: value })
    logger.debug('TrackingSettings', 'updated personalization and analytics consent')
  }
  const onPersonalizationChange = (value: boolean) => {
    updateUserConsentState({ personalization: value, productAnalytics })
    logger.debug('TrackingSettings', 'updated personalization consent')
  }
  const onProductAnalyticsChange = (value: boolean) => {
    updateUserConsentState({ personalization, productAnalytics: value })
    logger.debug('TrackingSettings', 'updated analytics consent')
  }

  return (
    <ScrollHeaderLayout.Default backgroundColor={colors.background20}>
      <View style={[styles.section, styles.borderRadiusBottom]}>
        <HoliContainer>
          <HoliGap size="s" />

          <TrackingGeneralContentDescription checkTextCountList={[1, 2, 4]} />

          <HoliGap size="s" />

          <View>
            {
              /* use arry of tuples where the elements in tuple are:
               * 1. text translation for the purpose
               * 2. value of the HoliSwitch
               * 3. onChange callback function for the HoliSwitch
               */

              (
                [
                  [
                    t('profile.settings.tracking.purpose.all'),
                    !!(personalization && productAnalytics),
                    onAllPurposesChange,
                  ],
                  [t('profile.settings.tracking.purpose.personalization'), !!personalization, onPersonalizationChange],
                  [t('profile.settings.tracking.purpose.analytics'), !!productAnalytics, onProductAnalyticsChange],
                ] as [string, boolean, (value: boolean) => void][]
              ).map(([text, value, onChange], idx) => (
                <View style={idx !== 0 && styles.optionBox} key={idx}>
                  <HoliBox
                    padding={[24, 0]}
                    layout={{ flexDirection: 'row', alignItems: 'center', justifyContent: 'space-between' }}
                  >
                    <HoliText>{text}</HoliText>

                    <HoliGap direction="horizontal" size="s" />

                    <HoliSwitch value={value} onChange={onChange} />
                  </HoliBox>
                </View>
              ))
            }
          </View>
        </HoliContainer>
      </View>

      <View style={[styles.section, styles.sectionGap, styles.borderRadiusTop, styles.borderRadiusBottom]}>
        <HoliContainer>
          <HoliBox padding={[24, 0]}>
            <HoliHeading level="3">{t('profile.settings.tracking.dataManagement.title')}</HoliHeading>

            <HoliGap size="sm" />

            <HoliText>
              <Trans t={t} i18nKey={'profile.settings.tracking.dataManagement.description'}>
                <HoliTextLink
                  label={t('profile.settings.help.support')}
                  href={isWeb ? t('global.support.mail.href') : ''}
                  onPress={isWeb ? undefined : () => Linking.openURL(t('global.support.mail.href'))}
                  color="brandB300"
                />
              </Trans>
            </HoliText>
          </HoliBox>
        </HoliContainer>
      </View>

      <HoliGap size="l" />
    </ScrollHeaderLayout.Default>
  )
}

export default memo(TrackingSettings)

const getStyles = (theme: HoliTheme) =>
  StyleSheet.create({
    section: {
      backgroundColor: theme.colors.background10,
    },
    sectionGap: {
      marginTop: 2,
    },
    borderRadiusTop: {
      borderTopLeftRadius: 24,
      borderTopRightRadius: 24,
    },
    borderRadiusBottom: {
      borderBottomLeftRadius: 24,
      borderBottomRightRadius: 24,
    },
    optionBox: {
      borderColor: theme.colors.background20,
      borderTopWidth: 1,
    },
  })
